#include <clib/alib_protos.h>
#include "DirTools.h"
#include "ListersGUI.h"
#include <mui/NlistView_mcc.h>

/* fill a FileInfoBlock with ExAllData */
void ExAll_2_Fib(struct ExAllData *ead,struct FileInfoBlock *fib)
{
    strcpy(fib->fib_FileName,ead->ed_Name   );  // Name of entry
    strcpy(fib->fib_Comment ,ead->ed_Comment);  // File comment

    fib->fib_DirEntryType   = ead->ed_Type;     // Type : file, dir or link
    fib->fib_Protection     = ead->ed_Prot;     // Access bits
    fib->fib_Size           = ead->ed_Size;     // File Size
    fib->fib_Date.ds_Days   = ead->ed_Days;     // Numbers of days since jan. 1, 1978
    fib->fib_Date.ds_Minute = ead->ed_Mins;     // number of minutes past Midnight
    fib->fib_Date.ds_Tick   = ead->ed_Ticks;    // number of ticks pas minute
    fib->fib_OwnerUID       = ead->ed_OwnerUID; // Owner's UID
    fib->fib_OwnerGID       = ead->ed_OwnerGID; // Owner's GID
}

/* fill list with file and directories */
void Refresh_DirList(APTR list)
{   struct ExAllControl *eac=NULL;
    BPTR lock=NULL;
    struct ExAllData EAData[EXALL_BUFFER_SIZE],*ead;
    struct FileInfoBlock fib;
    BOOL more;
    STRPTR path;

    if(!(path = malloc(MAX_PATH_TOTAL)))
    {   printf("Not enought memory !\n");
        return;
    }

    memset(path,0,MAX_PATH_TOTAL);
    get(list,MUIA_MyList_Path,&path);

    printf("[REFRESH_DIR_LIST] Got path \"%s\"\n",path);

    printf("[REFRESH_DIR_LIST] Lock path\n");
    if(!(lock = Lock(path,ACCESS_READ) ))
    {   printf("Could not get access to file or directory !\n");
        free(path);
        return;
    }

    printf("[REFRESH_DIR_LIST] Alloc Dos Object\n");
    if(!(eac = AllocDosObject(DOS_EXALLCONTROL,NULL)))
    {   printf("Could not allocate dos object !\n");
        free(path);
        UnLock(lock);
        return;
    }

    eac->eac_LastKey = 0;
    
    set(list,MUIA_List_Quiet,TRUE); // disable list drawing while adding entries
    printf("[REFRESH_DIR_LIST] Clear the List\n");
    DoMethod(list,MUIM_List_Clear);

    printf("[REFRESH_DIR_LIST] start directory listing \"%s\"\n",path);
    do
    {   more = ExAll(lock,EAData,EXALL_BUFFER_SIZE,ED_OWNER,eac);
        if((!more) && (IoErr() != ERROR_NO_MORE_ENTRIES))
            break; // anormal failure !

        if(eac->eac_Entries == 0)
        {   printf("[REFRESH_DIR_LIST] Directory is empty\n");
            continue; // normal failure: no entries
        }
        
        ead = EAData;
        do
        {   ExAll_2_Fib(ead,&fib);
            DoMethod(list,MUIM_List_InsertSingle,&fib,MUIV_List_Insert_Bottom);
            ead = ead->ed_Next;
        } while(ead);

    } while(more);

    printf("[REFRESH_DIR_LIST] Free ressources \n");
    if(eac) FreeDosObject(DOS_EXALLCONTROL,eac);
    UnLock(lock);
    if(path) free(path);
    printf("[REFRESH_DIR_LIST] Sort List\n");
    DoMethod(list,MUIM_List_Sort);
    printf("[REFRESH_DIR_LIST] Allow Drawing of items\n");
    set(list,MUIA_NList_Quiet,FALSE); // allow drawing list
    printf("[REFRESH_DIR_LIST] Directory refreshed successfully\n");
}

/* This function allow to set new path to a DirList obj */
BOOL SetNewPath(STRPTR Selection, APTR list,APTR STR_Path)
{
    STRPTR path = NULL;
    BOOL retval=TRUE;

    printf("[SET_NEW_PATH] Got \"%s\"\n",Selection);

    if(!(path = malloc(MAX_PATH_TOTAL)))
    {   printf("Not enought memory !\n");
        return FALSE;
    }
    memset(path,0,MAX_PATH_TOTAL);
    get(list,MUIA_MyList_Path,&path);

    printf("[SET_NEW_PATH] Current Path =  %s\n",path);

    if(strlen(path) + strlen(Selection) <= MAX_PATH_TOTAL)
    {   if(path[strlen(path)-1] == ':')
            strcat(path,Selection);
        else
            sprintf(path,"%s/%s",path,Selection);

        printf("[SET_NEW_PATH] Set new path to \"%s\"\n",path);

        set(list,MUIA_MyList_Path,path);
        set(STR_Path,MUIA_String_Contents,path);
    }
    else retval=FALSE;

    free(path);
    return retval;
}       

/* this function allow to go to parent directory of a DirList */
BOOL GoBack(APTR list,APTR STR_Path)
{
    int i=0,len=0;
    STRPTR path = NULL;
    BOOL retval=TRUE;

    if(!(path = malloc(MAX_PATH_TOTAL)))
    {   printf("Not enought memory !\n");
        return FALSE;
    }
    get(list,MUIA_MyList_Path,&path);

    printf("[GO_BACK] Current Path =  %s\n",path);

    i = len = strlen(path);

    // do something only if we are not already in a base dir //
    if(path[i-1] != ':')
    {
        printf("[GO_BACK] path[%i]!= ':' => we are not in root dir...\n",i-1);

        while( (path[i]!='/' && path[i]!=':')  && i>=0) i--;
         if(i>0)
         {   if(path[i] == ':') i++;
             memset(path+i,0,len-i);
             printf("[GO_BACK] new path set to \"%s\" %s\n",path);
             set(list,MUIA_MyList_Path,path);
             set(STR_Path,MUIA_String_Contents,path);
         }
         else retval=FALSE;
    }

    free(path);
    return retval;
}

void MoveFile_SameDevice(char *Buffer,char *src_path, char *dest_path, char *filename)
{   char Buffer2[MAX_PATH_SIZE];
    
    sprintf(Buffer,"%s%s",dest_path,filename);
    sprintf(Buffer2,"%s%s",src_path,filename);
    printf("[MOVEFILE_SAMEDEVICE] Moving %s to %s\n",Buffer2,Buffer);
    Rename(Buffer2,Buffer);
}

void MoveFile_OtherDevice(char *Buffer,char *src_path, char *dest_path, char *filename)
{
    sprintf(Buffer,"Copy %s%s TO %s ALL QUIET >NIL:",src_path,filename,dest_path);
    SystemTags((char *)Buffer,TAG_DONE);
    sprintf(Buffer,"Delete %s%s ALL QUIET >NIL:",src_path,filename);
    SystemTags((char *)Buffer,TAG_DONE);
}

void CopyFile(char *Buffer,char *src_path, char *dest_path, char *filename)
{
    sprintf(Buffer,"Copy %s%s TO %s ALL QUIET >NIL:",src_path,filename,dest_path);
    SystemTags(Buffer,TAG_DONE);
}

/* this function return strcmp() of device name only (eg: all char before ':') */
BOOL IsNotSameDevice(char* str1, char *str2)
{
    int i=0;
    while(str1[i]!=':') i++;
    return strncmp(str1,str2,i);
}

/* this function copy/move selected file of a list to given path */
void CopySelected(APTR list, char* DestBasePath, BOOL Delete)
{
    struct FileInfoBlock *entry=NULL;
    char *src_path=NULL,*Buffer=NULL;
    int len=0;
    LONG id = MUIV_List_NextSelected_Start;
    void (*Copyfunc) (char *Buffer,char *src_path, char *dest_path, char *filename);

    // Alloc needed memory //
    if( !(src_path=malloc(380)) ||
        !(entry=malloc(sizeof(struct FileInfoBlock)))) return;

    // Get base Source path //
    get(list,MUIA_MyList_Path,&src_path);
    len = strlen(src_path);
    if(src_path[len-1] != ':')
    {   src_path[len] = '/';
        src_path[len+1] = 0;
    }
    
    printf("[COPY_SELECTED] Got source path: \"%s\"\n",src_path);

    if((Buffer = malloc(len + strlen(DestBasePath) + MAX_PATH_SIZE + 25)))
    {   if(Delete)
            if(IsNotSameDevice(src_path,DestBasePath))
                 Copyfunc = MoveFile_OtherDevice;
            else Copyfunc = MoveFile_SameDevice;
        else Copyfunc = CopyFile;
        
        while(1)
        {   DoMethod(list,MUIM_List_NextSelected,&id);
            if(id==MUIV_List_NextSelected_End) break;
            DoMethod(list,MUIM_List_GetEntry,id,&entry);
            Copyfunc(Buffer,src_path,DestBasePath,entry->fib_FileName);
        }

        if(Delete) Refresh_DirList(list);
        free(Buffer);
    }

    free(entry);
    free(src_path);
}
